/**************************************************************************/
/*                                                                        */
/*   counter.c - Counter module to the VP7 Way Kool Project               */
/*   Copyright (C) 1999, 2000  VP7                                        */
/*                                                                        */
/*   Comments, questions and bugreports should be adressed to wkp@vp7.dk  */
/*   You can get the latest version of this program at http://wkp.vp7.dk/ */
/*                                                                        */
/*   This program is free software; you can redistribute it and/or modify */
/*   it under the terms of the GNU General Public License as published by */
/*   the Free Software Foundation; either version 2 of the License, or    */
/*   (at your option) any later version.                                  */
/*                                                                        */
/*   This program is distributed in the hope that it will be useful,      */
/*   but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/*   GNU General Public License for more details.                         */
/*                                                                        */
/*   You should have received a copy of the GNU General Public License    */
/*   along with this program; if not, write to the                        */
/*       Free Software Foundation, Inc.,                                  */
/*       59 Temple Place, Suite 330,                                      */
/*       Boston, MA  02111-1307                                           */
/*       USA                                                              */
/*                                                                        */
/**************************************************************************/

#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "counter.h"

#define MAX_DIGITS 9 /* MAA IKKE VAERE STOERRE END 9 */

static GtkWidget *digitwidget[MAX_DIGITS];
static GdkPixmap *digitpixmap[12];
static GdkBitmap *mask;

typedef struct _MyData{
  int digit[MAX_DIGITS];
  int hold;
  int reset;
  int mode;
  int power;
} MyData;

static MyData *pdata;

int integer_to_string (long int a, int str[], int i);

double calc_activity(int new_counts, double time_passed);

void initcounter(MyData *pdata);

char *filename( int n );

void power_callback( GtkWidget *widget, MyData *data );
void reset_callback( GtkWidget *widget, MyData *data );
void hold_callback( GtkWidget *widget, MyData *data );
void toggle_callback( GtkWidget *widget, MyData *data );


/* oversaetter a til en straeng med NUMBER_OF_DIGITS antal cifre.
   hoejeste ciffer er i str[NUMBER_OF_DIGITS-1], laveste i str[0] */
int integer_to_string (long int a, int str[], int i)
{
  if ( a < ( (long int) pow(10,i))) {
    while ( i > 0 )
      str[i] = (a % ((long int) pow(10,i)) ) / ( ((long int) pow(10,--i)) );
    return (TRUE);
  }
  else {
    printf("ERROR: Integer too large ( integer_to_string )\n");
    return (FALSE);
  }
}

double calc_activity(int new_counts, double time_passed)
{
  double activity;
  
  if (!(time_passed == 0)) {
    activity = new_counts / time_passed;
    /*printf(" %g \n",activity);*/
    return activity;
  }
  else
    { 
      printf("WARNING: division by zero ( activity )\n");
      return 0;
    }
}

int counter(int new_counts, double time_passed)
{
  static long int total_counts;
  int p=(TRUE); 
  int i;
  const int numberofdigits = MAX_DIGITS;
  int count_str[numberofdigits];
  int activity_str[numberofdigits];
  double activity;
  int reset, hold, mode, power;

  /* Griiiiiiiiimt! */
  reset=pdata->reset; hold=pdata->hold; mode=pdata->mode; power=pdata->power;
  activity=calc_activity(new_counts, time_passed);

  if (!power)
    for(i=0; i<numberofdigits; i++)
      /* hvis taelleren er slukket */
      gtk_pixmap_set(( GtkPixmap *)digitwidget[i], digitpixmap[11],mask);
  else {
     /* tael ikke, hvis hold og reset */
    if (!(reset && hold))
      total_counts = total_counts + new_counts;
    /* hvis reset og taelleroverloeb */
    if (total_counts>(pow(10,numberofdigits)-1) || (reset))
      total_counts=0;
    if (mode)
      p = integer_to_string((int) total_counts,count_str,numberofdigits);
    else
      p = integer_to_string((int) activity,activity_str,numberofdigits);
        
    if (((p && !hold) || reset) && mode)
      for(i=0; i<numberofdigits; i++)
	/* opdater widgits, er (forhaabentlig) globale*/
	gtk_pixmap_set(( GtkPixmap *)digitwidget[i], 
		       digitpixmap[(count_str[i])],mask);
    /* hvis activity mode .. */   
    if ((p || reset) && !mode) {
      gtk_pixmap_set(( GtkPixmap *)digitwidget[0], digitpixmap[10],mask);
      for(i=1; i<numberofdigits; i++)
	/* (reducer paranteser) */
	gtk_pixmap_set(( GtkPixmap *)digitwidget[i],
		       digitpixmap[(activity_str[i-1])],mask);
    }
  }
  if (!pdata->hold)
    pdata->reset = FALSE;
  return (p);
}

void initcounter( MyData *pdata )
{
  int q;

  pdata->hold = FALSE;  /* init hold button */
  pdata->reset = TRUE;  /* init counter & reset button*/
  pdata->mode = TRUE;   /* init mode button*/
  pdata->power = FALSE; /* init power button*/

  q=counter(0, 1);
  if (!q)
    /* ikke specielt sandsynligt, men pyt ... */
    printf("WARNING: counter not inititalized ( initcounter )");
  else printf("counter initialization completed.\n");
}

char *filename( int n ) /* Tak til alex@vp7.dk */
{
  static char *name[] = {
    "./pixmaps/0.xpm","./pixmaps/1.xpm","./pixmaps/2.xpm","./pixmaps/3.xpm",
    "./pixmaps/4.xpm","./pixmaps/5.xpm","./pixmaps/6.xpm","./pixmaps/7.xpm",
    "./pixmaps/8.xpm","./pixmaps/9.xpm","./pixmaps/enhed.xpm",
    "./pixmaps/dummy_digit.xpm"
  };
  return (n < 1 || n > 11) ? name[0] : name[n];
}

void power_callback( GtkWidget *widget, MyData *data )
{
  /* nulstil hvis apperat taendes */
  if (!(data->power)) {
    data->reset = TRUE;
    printf("Counter turned ON\n");
  }                              /* sri, Niels */
  else
    printf("Counter turned OFF\n");
  data->power = !(data->power);
}

void reset_callback( GtkWidget *widget, MyData *data )
{
  data->reset = TRUE;
}

void hold_callback( GtkWidget *widget, MyData *data )
{
  data->hold=!(data->hold);
}

void toggle_callback( GtkWidget *widget, MyData *data )
{
  int p;
  data->mode=!(data->mode); /* TRUE = counter; FALSE = activity ; */
  if ((data->mode) && (data->hold)) {
    data->hold= FALSE;
    p=counter(0,1);
    data->hold=TRUE;
  }  /* hvis hold er trykket og der skiftes fra aktiv.mode til norm.mode => updatertaeller*/ 
}

GtkWidget *create_counter (GtkWidget *window)
{
  /* internal variables*/ 
  int signed i;
  GtkWidget
    *box1, *box2, *box3, *mainbox,
    *resetbutton, *holdbutton, *togglebutton, *quitbutton,
    *frame, *digit, *label;
  GtkTooltips *tooltips;
  GtkStyle *style;

  pdata = g_malloc( sizeof(MyData));
  printf("Number of digits: %d\n",MAX_DIGITS);
  style = gtk_widget_get_style( window );
  /* load pixmaps */
  printf("Loading pixmaps...");
  for (i=0; i < 12; i++)
    /* load pixmaps */
    digitpixmap[i] = gdk_pixmap_create_from_xpm( window->window, &mask,
						 &style->bg[GTK_STATE_NORMAL],
						 filename(i) );
  printf("completed.\n");

  printf("Make pixmapswidgets...");
  /* create pixmapwidget to contain pixmaps */
  for (i=0; i < MAX_DIGITS; i++) {
    digitwidget[i] = gtk_pixmap_new( digitpixmap[i], mask );
    gtk_widget_show( digitwidget[i] );
  }
  printf("completed.\n");
  /* box for a label and the digits */
  box1 = gtk_hbox_new(FALSE, 0);
   
  label = gtk_label_new(" UniversalCounterBox \n by VP7.");
  gtk_box_pack_start(GTK_BOX(box1), label, TRUE, TRUE, 0);
  gtk_widget_show(label);
  /* add some digits to contain the pixmap widget */
  printf("Setup VP7 CounterBox(tm)...");
  for (i = (MAX_DIGITS-1); i > -1; i--) {
    digit =  gtk_event_box_new();
    gtk_container_add( GTK_CONTAINER(digit), digitwidget[i] );
    gtk_box_pack_start(GTK_BOX(box1), digit, TRUE, TRUE, 0);
    gtk_widget_show( digit );
  }
  gtk_widget_show(box1); /* finished box1 */
  
  box2 = gtk_vbox_new(FALSE, 0);  /* main box for box1 and box3 */
  gtk_box_pack_start(GTK_BOX(box2),box1, FALSE, FALSE, 0);
  gtk_widget_show(box1);
   /* button box */
  box3 = gtk_hbox_new(FALSE,0);
  gtk_box_pack_start(GTK_BOX(box2),box3,FALSE, FALSE,0);
  tooltips = gtk_tooltips_new (); 
  resetbutton = gtk_button_new_with_label ("  RESET  ");
  /* (se naeste knap) */
  gtk_signal_connect (GTK_OBJECT (resetbutton), "clicked",
		      GTK_SIGNAL_FUNC (reset_callback), (gpointer) pdata);
  gtk_box_pack_end(GTK_BOX(box3), resetbutton, FALSE, FALSE, 0);
  gtk_widget_show(resetbutton);
  gtk_tooltips_set_tip (tooltips, resetbutton, "Resets the display", NULL);

  holdbutton = gtk_toggle_button_new_with_label("  HOLD  ");
  /* <- se nedenstaaende kommentar */
  gtk_signal_connect (GTK_OBJECT (holdbutton), "clicked",
		      GTK_SIGNAL_FUNC (hold_callback), (gpointer) pdata);
  gtk_box_pack_end(GTK_BOX(box3), holdbutton, FALSE, FALSE, 0);
  gtk_widget_show(holdbutton);
  gtk_tooltips_set_tip (tooltips, holdbutton, "Freezes the display", NULL);

  togglebutton = gtk_toggle_button_new_with_label(" MODE ");
  /* <- dette argument giver vanskeligheder
     hvis jeg skriver MyData istedet for gpointer??*/
  gtk_signal_connect (GTK_OBJECT (togglebutton), "clicked",
		      GTK_SIGNAL_FUNC (toggle_callback), (gpointer) pdata);
  gtk_box_pack_end(GTK_BOX(box3), togglebutton, FALSE, FALSE, 0);
  gtk_widget_show(togglebutton);
  gtk_tooltips_set_tip (tooltips, togglebutton, 
			"Selects Counter/Activity mode", NULL);
  /* tooltips = gtk_tooltips_new ();  */
  quitbutton = gtk_toggle_button_new_with_label(" POWER ");
  gtk_signal_connect (GTK_OBJECT (quitbutton), "clicked",
		      GTK_SIGNAL_FUNC (power_callback), (gpointer) pdata);
  gtk_box_pack_start(GTK_BOX(box3), quitbutton, FALSE, FALSE, 0);
  gtk_widget_show(quitbutton);
  gtk_tooltips_set_tip (tooltips, quitbutton,
			"Turns the display ON/OFF", NULL);

  gtk_widget_show(box3);
  /* add a frame */
  frame = gtk_frame_new(NULL);
  gtk_frame_set_label( GTK_FRAME(frame), "VP7 CounterBox (tm)" );
  gtk_frame_set_label_align( GTK_FRAME(frame), 1.0, 0.0);
  gtk_frame_set_shadow_type( GTK_FRAME(frame), GTK_SHADOW_ETCHED_OUT);
  gtk_widget_show(frame);
  /* place main box in frame-container */
  gtk_container_add (GTK_CONTAINER (frame), box2);
  gtk_widget_show(box2);

  mainbox = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(mainbox), frame, FALSE, FALSE, 0);
  gtk_widget_show(mainbox);

  printf("completed.\n");
  printf("Calling initcounter()...");
  
  initcounter(pdata);

  return (mainbox);
}


/* ---eof--- */
